import os
from urllib.parse import urlparse

import psycopg
from psycopg import sql


def migrate() -> None:
    """
    Migrate the PostgreSQL database

    Watch for PostgreSQL version changes
    """
    url_parts = urlparse(os.environ["TODO_DB_DATABASE_URL"])

    # Extract connection parameters
    dbname = url_parts.path[1:]
    user_name = url_parts.username
    password = url_parts.password
    host = url_parts.hostname
    port = url_parts.port

    print(
        f"The database name is {dbname}, the username is {user_name}, "
        f"the password is {password}, the host is {host}, the port is {port}"
    )

    with psycopg.connect(
        dbname=dbname, user=user_name, password=password, host=host, port=port
    ) as conn:
        with conn.cursor() as cur:
            # create the members table
            cur.execute(
                sql.SQL(
                    """CREATE TABLE members (
                       id INT PRIMARY KEY GENERATED ALWAYS AS IDENTITY,
                       created TIMESTAMP NOT NULL DEFAULT now(),
                       email TEXT NOT NULL,
                       email_verified TIMESTAMP,
                       password_hash TEXT NOT NULL
                   )"""
                )
            )

            cur.execute(
                sql.SQL(
                    """CREATE UNIQUE INDEX members_unique_email_idx
                            ON members (LOWER(email)
                    )"""
                )
            )

            cur.execute(
                sql.SQL(
                    """CREATE TABLE todos (
                       id BIGINT PRIMARY KEY GENERATED ALWAYS AS IDENTITY,
                       complete BOOLEAN NOT NULL DEFAULT FALSE,
                       due TIMESTAMPTZ,
                       member_id INT NOT NULL REFERENCES members(id),
                       task TEXT NOT NULL
                   )"""
                )
            )

            # Commit the changes
            conn.commit()

    print("Finished PostgreSQL migration number 0")
    print("=================================================================")


if __name__ == "__main__":
    migrate()
